/* This work is licensed under the Creative Commons Attribution-Share Alike 2.5 Canada License.
 * This To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/2.5/ca/
 * This or send a letter to Creative Commons, 171 Second Street, Suite 300, San Francisco,
 * This California, 94105, USA.
 *
 * Written by John Simpson, Department of Philosophy, University of Alberta, 2009, 2010.
*/

#ifndef AGENT_H_
#define AGENT_H_

#include <map>
#include <bitset>
//#include <utility>
#include <gsl/gsl_rng.h>
#include <vector>
#include "game.h"

using namespace std;

class agent
{

	public:
		agent(const int, const int, const bool, const bool, const int ,const int, gsl_rng *, const int);
		~agent();

		//Determines an agent's move given a particular game and partner
		//Moves are relative to status quo given by the game
		//True = "play on status quo"
		//False = "play off status quo"
		bool play (game *const, const int, const bool);

		int  agentID (); //returns the agent's ID on request

		void updateAgent (game *const, const int, int, int, const bool );

		void printAgent();

		int agentRes(); //returns resources held by the agent

		void setRes(int); //sets an agent's resources to the passed int

		void agentSpawn(agent*, agent*, gsl_rng *); //spawns a new agent and returns a pointer to this agent

		bool choiceBitTest(int); //Tests a bit in the agents choice bitset

		unsigned char afterPlayChar(const int, const int, const int); //Returns an indexed char from afterPlay

		int agentAge(); //returns agent age

		void agentAgePlus(); //adds ONE to agent age

		int agentBirthday(); //Returns the agent's birthday

		int agentBitsSet(); //Returns number of bits set in choice

		int afterPlayValues(const int, const int, const int); //Returns a value corresponding to one of the three afterplay settings

		int choiceSize(); //returns size of choice bitset

		bool spawnAgree(const int); //says whether an agent is willing to spawn with another

		bool spawnPrefSet(const int); //returns true if a specific bit in spawnPref is set

		bool playAgree(const int);  //says whether an agent is willing to pplay with another

		bool playPrefSet(const int);  //returns true if a specific bit in playPref is set

		unsigned char getDefaultMLU();

		unsigned char getDefaultULM();

		bool agentKnow(const int); //returns whether or not a given agent is known

		short agentLike(const int, const bool); //allows specific likings for agents in know to be discovered for arc creation

		int agentsKnown(); //returns the number of agents the agent has likings for

		int likingStatsCount(const int);  //returns the number of likings of a given type an agent has

		void cleanKnow(vector<int>*, const int); //Updates know to reflects shift to a new generation and removes old agent IDs

	private:

		//testing value to track number of bits set for each agent
		int bitsSet;

		// id=idCount on the creation of each agent
		int id;

		//the generation number in which the agent first started interacting with other agents,
		short birthday;

		// number of generations an agent has been alive
		short age;

		// CURRENTLY UNUSED AS AGENTS ARE STRAIGHT(ISH)1 MAXIMIZERS
		// sqCode is a representation of an agent's interpretation of the
		// STATUS QUO of a given situation.  That is, as the game is
		// currently presented what would the agent expect to happen if
		// neither they nor the other player changed from the default
		// strategies.  When created an agent looks at the difference
		// between the payoff at the status quo position AND the payoff
		// if they were to unilaterally switch.  Differences in these
		// help set a predisposition to switch or stay.
		//unsigned char sqCode[10][10];

		/* This matrix captures how an agent will take various outcomes following
		 * the playing of a game and the assignment of payoffs.  The agent updates
		 * how much they like the other agent based on the combination of whether
		 * or not they do better, worse, or the same from the status quo in
		 * combination with how the other player does wrt the status quo as well.
		 * Unsigned chars are used to save space over ints.
		 * The first index value says how the agent fared against the status quo.
		 * The second index value says how the other agent fared against the status quo.
		 * The third index value says what the agent thought of the other agent prior to playing
		 * There are three possible values that can appear in the array:
		 * F - make opponent a "friend"
		 * N - make opponent a "neutral party" (Note that any non F or E value is counted as an N)
		 * E - make opponenet an "enemy"
		*/
		unsigned char afterPlay[3][3][3];

		/* If agents know anything about their opponent it is represented in a map.  This map takes the
		 * opponent's ID as the key and returns a bitset (used to save space).  An agent for which there is
		 * no knowledge will not be included.  This was originally stored as a bit set but it proved too hard to
		 * manage.
		 * map <key=otherAgentID, pair< pair<meLikesYou, youLikesMe>, gens since not seeing agent with ID>
		*/
		map<unsigned int, pair<pair<unsigned char, unsigned char>, unsigned short> > know;


		/* The variable maxAbsent sets how many generations are allowed to pass before an
		 * agent is dropped from an agent's know.  This is critical for reducing memory load
		 * on long living/large populations.
		*/

		unsigned short maxAbsent;

		/*
		Finally, all this information needs to come together in a way
		that allows a choice to be made.  This will be done using a bitset
		that completely captures what each agent would do on any possible
		presentation of variables.  The bitset wil be treated as an array
		with the following dimensions:

		5x5x2x2x2x2x2x2x2x2x2x2x2x2

		1 = How much the agent likes the other player [0-4]
		2 = How much the agent thinks the other player likes him [0-4]
		3 = Is there a better outcome for the agent if they unilaterally switch [0-1]
		4 = Is there a worse outcome for the agent if they unilaterally switch [0-1]
		5 = Is there a better outcome for the agent if the other player unilaterally switches [0-1]
		6 = Is there a worse outcome for the if the other player unilaterally switches [0-1]
		7 = Is there a better outcome for the agent if they both switch [0-1]
		8 = Is there a worse outcome for the agent if they both switch [0-1]
		9 = Is there a better outcome for the other if they unilaterally switch [0-1]
		10 = Is there a worse outcome for the other if they unilaterally switch [0-1]
		11 = Is there a better outcome for the other if the agent unilaterally switches [0-1]
		12 = Is there a worse outcome for the other if the agent unilaterally switches [0-1]
		13 = Is there a better outcome for the other if they both switch [0-1]
		14 = Is there a worse outcome for the other if they both switch [0-1]

		This is not the most efficient way to hold this information. Take,
		for example, how bits 3 & 4 relate to each other.  If 3 is "on"
		then it would seem to be a logical contradiction for 4 to be "on"
		as well (If both are "off" then this would simply indicate that
		the agent believes that they do just as well switching as staying.
		A 3-valued system seems to be more appropriate in this case since
		having two yes-no questions requires four states to capture all
		the possible outcomes whereas a single question with three possible
		answers would be more compact.  Carried over all the questions
		this would reduce the number of possible combinations from 4096
		(2^12) to 729 (3^6), saving a great deal of space and construction
		time.

		However, accepting this argument requires overlooking that there
		are two possible ways that one might be indifferent, each of which
		may affect an agent differently.  The first is the way described
		earlier: the agent attaches the same value to all options.  The
		second occurs when the agent does not have a clear understanding
		of how the possiblities stand in relation to each other; thus an
		option may appear to be both better and worse.  Cases such as this
		seem to occur frequently in decision making when the outcomes are
		not homogeneous, possibly including unknowns in their composition.
		However, as currently constructed these agents are unable to
		actually face a problem with heterogenous outcomes.

		Hmmm.  It occurs to me that yes, they do face problems with
		heterogenous outcomes if the agent includes both their payoff AND
		the other player's payoff in their evaluation of the outcome, as
		they are currently doing.  Thus the heterogeneity is introduced
		by having the agent differentiate between their own well-being
		and the well-being of the other player.  This creates NINE
		possible situations for every game.  By switching from the status
		quo:
		Everyone does better
		Everyone does worse
		Everyone does the same
		I do better, but you do worse
		I do better, but you do the same
		I do the same, but you do worse
		I do the same, but you do better
		I do worse, but you do the same
		I do worse, but you do better
		It is these latter SIX combinations that count as heterogeneous.
		This is further compounded by the ways that we might like each
		other combine.

		Update - Tuesday, February 2, 2007
		Five possible ways to like some one seems a bit much to manage
		in regards to setting thresholds one which an agent shifts
		perceptions of other agents around.  Five kinds of liking would
		require four thresholds and this would be quite messy to track and
		update.  Thus I will settle for 3/4 kinds of liking, as follows:
		1. The case where I do not know you at all.
		2. The case where I know you and like you.
		3. The case where I know you and do not like you.
		4. The case where I know you, but am uncertain of how I like you.

		Only 8192 bits are needed now (3^2*2^12)
		Was 36864 after dumping reducing four ways of liking to three
		Then discovered that most of these code for junk
		(i.e. agent can never assess a move as BOTH better AND worse)
		so reduced to 6561.  A map from bit code to place in bitset
		is needed to correctly pull agent choice.
		*/
		bitset<6561> choice;

		/*
		 * The following variable tracks the sum of payoffs that an agent
		 * receives over the course of their game playing less antes and spawning costs
		*/
		int resource;

		/* Controls whether or not an agent will spawn with other agents depending on how
		 * those agents are liked.  First bit sets for 'E' second for 'N' third for 'F'.
		 * A set bit means an agent will spawn with agents with that property
		*/
		bitset<9> spawnPref;

		bitset<9> playPref;

		//The default liking value that an agent chooses when
		//facing an unknown agent
		unsigned char dmLu, duLm;

		int bit2pos(const int);
		int pos2bit(const int);

		/*Tracks how many agents the current agent thinks are friends etc.
		 * 0 mLu==N
		 * 1 mLu==E
		 * 2 mLu==F
		 * 3 uLm==N
		 * 4 uLm==E
		 * 5 uLm==F
		*/
		int likingStats[6];
};

#endif /*AGENT_H_*/
