/* This work is licensed under the Creative Commons Attribution-Share Alike 2.5 Canada License.
 * This To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/2.5/ca/
 * This or send a letter to Creative Commons, 171 Second Street, Suite 300, San Francisco,
 * This California, 94105, USA.
 *
 * Written by John Simpson, Department of Philosophy, University of Alberta, 2009, 2010.
*/

#ifndef GAME_H_
#define GAME_H_

#include <bitset>
#include <utility>
#include <gsl/gsl_rng.h>

using std::bitset;

//Builds a game with random payoffs from 0 to 9
class game
{
	friend class agent;

	public:
		game(const int, gsl_rng *);
		~game();

		int outcome (const bool, const bool, const bool); //Determines outcome given agent choices
		void printGame(); //Prints the game matrix, and status quo postion
		int payBits(const int); //Returns the payoff for a particular player outcome combination
		int getPaySum(); //Returns sum of all possible payoffs for the game

	private:
		int gameType;

		void RNDgame(gsl_rng *); //Builds a random game
		void PDgame(gsl_rng *); //Builds a Prisoner's Dilemma
		void HNTgame(gsl_rng *); //Builds a Stag Hunt
		void CHKgame(gsl_rng *); //Builds a Chicken
		void BSgame(gsl_rng *); //Buils a Battle of the Sexes

		/* This is the standard 2x2 game matrix with 4 outcomes and 8 payoffs. */
		/*
		Convention:
		pays[0]=payoff to player A for outcome T,L
		pays[1]=payoff to player A for outcome B,L
		pays[2]=payoff to player A for outcome T,R
		pays[3]=payoff to player A for outcome B,R
		pays[4]=payoff to player B for outcome T,L
		pays[5]=payoff to player B for outcome T,R
		pays[6]=payoff to player B for outcome B,L
		pays[7]=payoff to player B for outcome B,R
		*/
		int pays[8];

		//Sum of all available payoffs in the game
		int paySum;

		/* This bitset says what the status quo cell for the game,
		 * as seen by the players, will be.
		 * It is assumed both players share the same perspective.
		 * 00=bottom right
		 * 01=bottom left
		 * 10=top right
		 * 11=top left
		*/
		bitset<2> sq;

		/* This bitset says what kind of game is being played based on
		 * better/worse unilateral/mutual switching.
		 * Bit order is as follows:
		 * Does row do better or worse if they switch and column stays?
		 * Better?  Worse?
		 * Does row do better or worse if column unilaterally switches?
		 * Does row do better or worse if they both switch?
		 * Does column do better or worse if they switch and row stays?
		 * Does column do better or worse if row switches and they stay?
		 * Does column do better or worse if they both switch?
		*/
		bitset<12> bw;

		/* Carries the status quo payoffs for easy access
		 * The first in the array is the Row Player's payoff.
		 * The second is the Column Player's payoff.
		*/
		int sqPays[2];

		/* This bitset says what the outcome of the game is with reference to the status quo
		 * position.
		 * 00 = both played on status quo
		 * 01 = "Player 1" played off.
		 * 10 = "Played 2" played off
		 * 11 = both played off
		*/
		bitset<2> out;
};

/*
//Builds a game with a random PD payoff structure
class PDgame : public game
{
	PDgame(void);
	~PDgame();

	int outcome (const bool, const bool, const bool);
	int  payoff (bool, bool, bool);
	void rowColumn (int, int);
	void printGame();
	int payBits(const int);

	private:

		int pays[8];
		bitset<2> sq;
		bitset<12> bw;
		bitset<2> out;

};

//Builds a game with a random Chicken payoff structure
class CHKgame : public game
{
	CHKgame(void);
	~CHKgame();

	int outcome (const bool, const bool, const bool);
	int  payoff (bool, bool, bool);
	void rowColumn (int, int);
	void printGame();
	int payBits(const int);

	private:

		int pays[8];
		bitset<2> sq;
		bitset<12> bw;
		bitset<2> out;
};

//Builds a game with a random Stag Hunt payoff structure
class HNTgame : public game
{
	HNTgame(void);
	~HNTgame();

	int outcome (const bool, const bool, const bool);
	int  payoff (bool, bool, bool);
	void rowColumn (int, int);
	void printGame();
	int payBits(const int);

	private:

		int pays[8];
		bitset<2> sq;
		bitset<12> bw;
		bitset<2> out;
};

//Builds a game with a random Battle of the Sexes payoff structure
class BSgame : public game
{
	BSgame(void);
	~BSgame();

	int outcome (const bool, const bool, const bool);
	int  payoff (bool, bool, bool);
	void rowColumn (int, int);
	void printGame();
	int payBits(const int);

	private:

		int pays[8];
		bitset<2> sq;
		bitset<12> bw;
		bitset<2> out;
};
*/

#endif /*GAME_H_*/
