/* SR6 Version Highlights:
 * Random seed resetting and incrementing to allow same initial population to experience different conditions
 * Pull first population before anything happens
 * Pull at end of first generation and every fifth generation
 * Pull at last population, either due to explosion, death, or time
 * OpenMP (Number of processors must be specified just after first while loop.  File locking is done through
 * 			critical pragma that encapsulates each fbit of file writing.
 * Use of the Gnu Scientific Library Random Number Generator (gsl_rng.h) in place of Vance's RNG.
 * 			The GSL RNG is thread safe and easier to track and invoke.
 * To Do:
 * Collect redundant information in In.dat files into the first line and move all variables immediately before they are used.
 * Build games after matching agents so that games are built only for the number of agents matched.  This will save memory and increase speed.
 *
 * This work is licensed under the Creative Commons Attribution-Share Alike 2.5 Canada License.
 * This To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/2.5/ca/
 * This or send a letter to Creative Commons, 171 Second Street, Suite 300, San Francisco,
 * This California, 94105, USA.
 *
 * This work is licensed under the Creative Commons Attribution-Share Alike 2.5 Canada License.
 * This To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/2.5/ca/
 * This or send a letter to Creative Commons, 171 Second Street, Suite 300, San Francisco,
 * This California, 94105, USA.
 *
 * Written by John Simpson, Department of Philosophy, University of Alberta, 2009, 2010.
*/

/*Libraries that I needed (or thought I needed) at one point or another, but no longer need. */
//#include <string>
//#include <memory>
//#include <cstdlib>
//#include <utility>
//#include <stdio.h>

#include <iostream>
#include <iomanip>
#include <fstream>
#include <map>
#include <bitset>
#include <vector>
#include <algorithm>
#include <omp.h>
#include <gsl/gsl_rng.h>
#include "agent.h"
#include "game.h"
#include "match.h"

using namespace std;

//Sets up header formatting for output files
void prepOutfiles(ostream&, ostream&, ostream&, const int, const int, const int); //Sets up header formatting for output files

int bit2pos(const int);
int pos2bit(const int);
int PDbitBase(const int);
int SHbitBase(const int);
int CKbitBase(const int);
int BSbitBase(const int);

//cleans out the weak and distributes random death
void cull(vector<agent*>*, int &, const int, const float, vector<int>*, map<unsigned int, pair<unsigned short, unsigned short> >*, const bool, gsl_rng *);

//regenerates the new population based on who is left and what they have
void breed(vector<agent*>*, int &, const int, const int, const int, const bool, const bool, const int, const int, gsl_rng *, int &);

//collects and prints full population statistics
void statsSummary(const int, vector<agent*>*, const int, const int, const int, const int, ostream&, ostream&, ostream&, ostream&, const bool);

//collects and prints only population size statistics
void popStatsSummary(const int, vector<agent*>*, const int, const int, const int, const int, ostream&);

int main(void)
{
	//Declare and initialize background control variables
	int ante=34; 				//number of resource points necessary to survive at the end of a generation
	float rndDeathRate=0.04; 	//Probability that any given agent will die, regardless of resources held
	int spawnCost=9; 			//Per Agent Cost to Spawn
	int g=1; 					//Sets game type for ALL rounds. 0: Random, 1: PD , 2:Stag Hunt, 3: Chicken, 4: Battle of the Sexes
	int runType=0;				//Controls what data is collected.  0: Pull only population stats for fast population assessments 1: Pull Full stats summaries.
	int numAgents=100;			//Number of Agents to create at the start of trial
	int numPops=50;				//Number of populations to test across different environmental seeds
	int numEnvs=50;				//Number of environments to test each population across
	int numGens=100;			//Number of generations within a population
	int numRounds=10;			//Number of rounds per generation
	int spawnDisc=0;			//If 0 then indiscriminate spawning.  If >0 then indicates number of spawning attempts allowed per generation
	int playDisc=0;				//If 0 then indiscriminate game playing.  If >0 then indicates the number of attempts allowed per round
	bool rndAft=false;			//Sets randomization in agent afterplay behaviour.  False sets N for all outcomes.
	bool rndLike=false;			//Sets randomization across agents of how unknown agents are initially liked
	int maxAgents=10000;		//Number of living agents at which a population ceases to run
	int maxAbsent=numRounds;	//Number of rounds not encountering an agent after which that agent is forgotten
	int choiceSize=6561; 		//size of choice bitset
	string outFileName;			//holds the name of the output file
	bool socNet=false;			//When TRUE the simulation collects social networking information in additional outFiles.
								//The socNet  files are always created, but only filled is socNet is on.
	int trialLineNum=0;			//Says which line of In.dat the master data was pulled from.  Also used to differentiate between different runs.
	//float bitThreshold=0.95; 	//Reporting threshold for bit convergence - NOT currently implemented

	//Open input file with ifstream constructor
	ifstream inFile( "In.dat", ios::in);

	//exit program if input file failure
	if (!inFile)
	{
		cerr << "IN.DAT COULD NOT BE OPENED" << endl;
		exit (1);
	}  //end if

	//ofstream constructor opens files
	//RT-#0.csv carries all the summary data
	//RT-#1.csv & RT-#2.csv carry all the percentage distributions for bits set in CHOICE that can actually
	//influence games given that agents cannot assess outcomes as being BOTH better and worse.
	//There are multiple files so as not to violate ASCII line length restrictions.  These can all be stitched back together in Stata.
	//This is set-up to handle file sharing on the cluster via naming each file specifically, but this is not currently implemented.

	outFileName="RT-#0.csv";
	ofstream outFile0(outFileName.data(), ios::out);
	outFileName="RT-#1.csv";
	ofstream outFile1(outFileName.data(), ios::out);
	outFileName="RT-#2.csv";
	ofstream outFile2(outFileName.data(), ios::out);

	//exit if unable to open a file
	if ( !outFile0 || !outFile1 || !outFile2) //overloaded ! operator
	{
		cerr << "AN OUTPUT FILE COULD NOT BE OPENED" << endl;
		exit(1);
	}

	//Pulls the type of game to be used for first set of trials.  This SHOULD be the type used for all trials
	inFile >> runType >> socNet >> numAgents >> maxAgents >> maxAbsent >> g >> ante >> spawnCost;

	//Prepare the files which will hold the outputs of the simulation
	prepOutfiles(outFile0, outFile1, outFile2, choiceSize, g, runType);

	//Reset the infile "get pointer" to the beginning.
	inFile.seekg(0);

	/*set thread tracking variables*/
	int nthreads=1;		//total number of thread used
	int tid=0;			//tid stores the # of each thread
	int npt=0; 			//np is used to control rng resetting. npt keeps np on entry safe from threads


//This is the loop that runs the populations listed in IN.DAT
while (inFile >> runType >> socNet >> numAgents >> maxAgents >> maxAbsent >> g >> ante >> spawnCost >> rndDeathRate >> spawnDisc >> playDisc >> rndLike >> rndAft >> numPops >> numEnvs >> numGens >> numRounds)
{

(void) omp_set_num_threads(4);
#pragma omp parallel for firstprivate(spawnDisc,playDisc,rndLike,rndAft,runType,socNet,numAgents,maxAgents,maxAbsent,g,ante,spawnCost,rndDeathRate,numPops,numEnvs,numGens,numRounds,tid,npt)
for (int np=0; np<numPops; np++)
{
	int deadGen=0; 				//Tracks number of dead at end of each generation
	int bornGen=0; 				//Tracks the number of agents born into each new generation
	int paySumGen=0; 			//Sum of all possible payoffs over a generation
	int paidSumGen=0; 			//Total of payoffs actually received
	int gamesGen=0; 			//Total number of games played over a generation

	npt=np;
	/* Obtain thread ids and print number of threads */
	tid = omp_get_thread_num();

	//Critical pragma confines writing of the file to one process
	#pragma omp critical (output_data)
	{
		if (tid == 0)
		{
			nthreads = omp_get_num_threads();
			printf("Number of threads = %d\n", nthreads);
		}
	}

	gsl_rng * r = gsl_rng_alloc (gsl_rng_mt19937);  //set-up random number generator

for (int ne=0; ne<numEnvs; ne++)
{
	int agentCount=0;	//Used to track number of agents created and to pass agents IDs

	//Initialize the vector	that will contain POINTERS TO agents
	vector<agent*> allAgents;

	gsl_rng_set (r, (ante/5-70)*10000000+(spawnCost/25-10)*1000000+trialLineNum*10000+npt*100+ne);

	//Restart RNG with same initial seed for each
	//gsl_rng_set (r, npt);

	//Critical pragma confines writing of the file to one process
	#pragma omp critical (output_data)
	{
		//cout << "Thread " << tid << " is beginning Trial " << trialLineNum << ", Population " << npt << ", Environment " << ne << endl;
		//cout << gsl_rng_get(r) << endl;
	}

	//Create the appropriate number of agents and push POINTERS TO EACH into a vector
	for (int i=0; i<numAgents; i++)
	{
		agentCount++;
		allAgents.push_back(new agent(spawnDisc, playDisc, rndLike, rndAft, 1, maxAbsent, r, agentCount));
	}

	//cout << gsl_rng_get(r) << endl;

	//Increment seed so future interactions are not identical to previous runs with this population
	//gsl_rng_set (r, ne);

	//cout << gsl_rng_get(r) << endl;


	//The next two outfiles are used to store the social network information from generation to
	//generation.  Vertices.txt holds information on when agents were alive.  Arcs.txt holds
	//the information on the nature of the relationships between agents. Not set up for OMP

	//string popStr = static_cast< string >(np);
	outFileName="PajekFile"; // + (string)np;
	ofstream pajFile(outFileName.data(), ios::out);
	outFileName="Arcs.txt";
	ofstream arcFile(outFileName.data(), ios::out);
	arcFile << "*Arcs\r" << endl;


	//Take Snap Shot of Initial Population
	//Send basic information to summary file
	#pragma omp critical (output_data) //Critical pragma confines writing of the file to one process
	{
	outFile0 << trialLineNum << "," << npt << "," << ne << "," << g << "," << ante << "," << spawnCost << "," << rndDeathRate << ",";
	outFile0 << spawnDisc << "," << playDisc << "," << rndLike << "," << rndAft << "," << numAgents << "," << numGens << ",";
	outFile0 << numRounds << "," << -1 << "," << "," << ",";

	switch (runType)
	{
		case 0:
		{
			popStatsSummary(-1, &allAgents, 0, 0, choiceSize, g, outFile0);
			break;
		}
		case 1:
		{
			statsSummary(-1, &allAgents, 0, 0, choiceSize, g, outFile0, outFile1, outFile2, arcFile, socNet);
			break;
		}
	}
	} // End critical pragma output_data

	//Create the map that will link agent IDs to the birth and death times for each agent.  Needed for timed
	//social network visualization with Pajek
	map<unsigned int, pair<unsigned short, unsigned short> > vertTimes;


	//This is the generation loop
	//NOTE: break statement near end to prevent memory overload
	for (int gen=0; gen<numGens; gen++)
	{
		/*
		//Print visual tracking information on screen
		#pragma omp critical (print_Geninfo) //Critical pragma confines writing of the file to one process
		{
		cout << endl << "------------------ GENERATION " << gen+1 << "- Thread# " << tid << " - Trial Number# " << trialLineNum;
		cout << " Population # " << np+1 << "/" << numPops << " ------------------"<< endl << endl;
		cout << allAgents.size() << " agents are alive at the beginning." << endl;
		} // End critical pragma
		*/

		//set/reset generation tracking variables
		paySumGen =0;
		paidSumGen = 0;
		gamesGen = 0;

	for (int round=0; round<numRounds; round++)
	{
		//BUILD GAMES

		//Calculate the number of games necesary for each agent to play
		int numGames=allAgents.size()/2;

		//cout << "Generating " << numGames << " different games." << endl << endl;

		//Build the games that agents will play (this round) and contain
		//them in a vector for easy storage and indexing
		vector<game*> allGames;

		//Build the games that agents will play (this round) and contain them in the vector allGames
		for(int i=0; i<numGames; i++)
			allGames.push_back(new game(g, r));

		//MATCH PLAYERS

		//Randomly match players and store these matches in a vector
		//that holds pairs of Player ID#s;
		vector<match> allMatches;

		//The vector of agent ID#s will be populated with all the ID#s in
		//allPlayers.  ID#s will be randomly pulled from here and then
		//placed in a pair inside allMatches to ensure no double plays
		vector<pair<unsigned long, unsigned short> > agentIndex;

		pair<unsigned long, unsigned short> tempPair;

		for (int i=0; i<(int)(allAgents.size()); i++)
		{
			tempPair.first=i;
			tempPair.second=0;
			agentIndex.push_back(tempPair);
		}

		//Build a set of matches by randomly pulling ID#s from agentIndex.
		//tempMatch is to hold index values temporarily before storing
		//them in allMatches
		match tempMatch;

		int index1=0;
		int index2=0;
		bool matchFlag=false;

		while (agentIndex.size()>=2)
		{
			//cout << "LOOPING!!! Number of agents= " <<  agentIndex.size();
			index1 = gsl_rng_uniform_int(r,agentIndex.size());
			index2 = gsl_rng_uniform_int(r,agentIndex.size());
			matchFlag=false;

			if (index1!=index2) //Prevents play attempt from incrementing if an agent was scheduled to play themselves
			{
				agentIndex.at(index1).second++;
				agentIndex.at(index2).second++;

				if (playDisc<=0
					|| (playDisc>0
					&& ((allAgents.at(agentIndex.at(index1).first))->playAgree(allAgents.at(agentIndex.at(index2).first)->agentID())==true
					&& (allAgents.at(agentIndex.at(index2).first))->playAgree(allAgents.at(agentIndex.at(index1).first)->agentID())==true)))
				{
					//Assigns the index values of the players for one game to the allAgents vector
					tempMatch.assignment(1, agentIndex.at(index1).first);
					tempMatch.assignment(0, agentIndex.at(index2).first);
					allMatches.push_back(tempMatch);
					matchFlag=true;
				}

				if (index1 > index2)
				{
					if (matchFlag==true || agentIndex.at(index1).second >= playDisc)
					{
						//cout << "agent ID" << allAgents.at(agentIndex.at(index1).first)->agentID() << " removed" << endl;
						agentIndex.erase( agentIndex.begin() + index1);
					}
					if (matchFlag==true || agentIndex.at(index2).second >= playDisc)
					{
						//cout << "agent ID" << allAgents.at(agentIndex.at(index2).first)->agentID() << " removed" << endl;
						agentIndex.erase( agentIndex.begin() + index2);
					}
				}
				else
				{
					if (matchFlag==true || agentIndex.at(index2).second >= playDisc)
					{
						//cout << "agent ID" << allAgents.at(agentIndex.at(index2).first)->agentID() << " removed" << endl;
						agentIndex.erase( agentIndex.begin() + index2);
					}
					if (matchFlag==true || agentIndex.at(index1).second >= playDisc)
					{
						//cout << "agent ID" << allAgents.at(agentIndex.at(index1).first)->agentID() << " removed" << endl;
						agentIndex.erase( agentIndex.begin() + index1);
					}
				}
			}
		}

		/*
		//Output Match Information on screen
		for(int i=0; i<(int)(allAgents.size()/2); i++)
		{
			cout << "Matched Pair " << i << endl;
			allMatches.at(i).printMatch();
		}
		*/

		//MATCH PLAYER PAIRS TO GAME AND PLAY

		int 	rowID=0;
		int 	colID=0;
		bool 	rowPlay=-1;
		bool 	colPlay=-1;
		int 	rowPay=-1;
		int 	colPay=-1;

		for (int i=0; i<(int)allMatches.size(); i++)
		{
			/*look in allMatches to get location in allAgents of each player
			look in allAgents and have each player in the pair play the next
			game in allGames
			The game takes these plays and determines the outcome, passing it
			to the players which then update themselves.
			By convention the FIRST PLAYER in the matched pair is ALWAYS
			the ROW PLAYER*/

			/*//Print Games information on screen
			cout << "The following game is being played..." << endl << endl;
			allGames.at(i)->printGame();

			cout << "...by the following agents..." << endl;
			*/

			rowID=allAgents.at(allMatches.at(i).firstInPair())->agentID();
			colID=allAgents.at(allMatches.at(i).secondInPair())->agentID();
			//allMatches.at(i).printMatch(rowID, colID);

			rowPlay=allAgents.at(allMatches.at(i).firstInPair())->play(allGames.at(i), colID, true);
			colPlay=allAgents.at(allMatches.at(i).secondInPair())->play(allGames.at(i), rowID, false);

			rowPay=allGames.at(i)->outcome(true, rowPlay, colPlay);
			colPay=allGames.at(i)->outcome(false, rowPlay, colPlay);

			gamesGen++;

			/*//Print final outcome of each game on screen.
			cout << "The Row Player (Agent# " << rowID << ") plays ";
			if (rowPlay==true)
				cout << "ON ";
			else
				cout << "OFF ";
			cout << "the status quo." << endl;

			cout << "The Column Player (Agent# " << colID << ") plays ";
			if (colPlay==true)
				cout << "ON ";
			else
				cout << "OFF ";
			cout << "the status quo." << endl;

			cout << "Payoff to Row Player on Outcome: " << rowPay << endl;
			cout << "Payoff to Column Player on Outcome: " << colPay << endl << endl;
			*/

			paySumGen = paySumGen + allGames.at(i)->getPaySum();
			paidSumGen = paidSumGen + rowPay + colPay;

			//Update agents based on outcome
			allAgents.at(allMatches.at(i).firstInPair())->updateAgent(allGames.at(i), colID, rowPay, colPay, true);
			allAgents.at(allMatches.at(i).secondInPair())->updateAgent(allGames.at(i), rowID, colPay, rowPay, false);
		}

		//Clean-up things not required in a possible future round:
		//allGames, allMatches, agentIndex
		agentIndex.clear();
		allMatches.clear();

		for (int i=0; i<(int)allGames.size(); i++)
			delete allGames.at(i);
		allGames.clear();

	}  //Closes the loop that cycles through each ROUND

		//Collect all dead agent IDs so that they can be
		//removed from remaining agent memories to conserve space
		vector<int> deadIDs;

		//Remove agents who have failed to meet the ante
		cull(&allAgents, deadGen, ante, rndDeathRate, &deadIDs, &vertTimes, socNet, r);
		//cout << deadGen << " agents die." << endl;

		//Increment age of surviving agents
		if (numGens-gen>1)
			for (int i=0; i<(int)(allAgents.size()); i++)
				allAgents.at(i)->agentAgePlus();

		//Allow surviving agents to breed if they have sufficient resources
		breed(&allAgents, bornGen, spawnCost, spawnDisc, playDisc, rndLike, rndAft, gen+1, maxAbsent, r, agentCount);
		//cout << bornGen << " agents are spawned by remaining agents." << endl;

		//Summarize statistics and send to file for first gen, every 5 gens, if pop explodes, or crashes to <=1)
		#pragma omp critical (output_data) //Critical pragma confines writing of the file to one process
		{
		//Turning on the IF immediately below calls the stat summary right at intervals rather that every time
		//if ((gen % 5==0) || gen==1 || (int)allAgents.size()>maxAgents || allAgents.size()<=1)
		//{
			//Send basic information to summary file
			outFile0 << trialLineNum << "," << np << "," << ne << "," << g << "," << ante << "," << spawnCost << "," << rndDeathRate << ",";
			outFile0 << spawnDisc << "," << playDisc << "," << rndLike << "," << rndAft << "," << numAgents << "," << numGens << ",";
			outFile0 << numRounds << "," << gen << ",";

			//Send Average Payout Offered and Average Payoff Received to File
			if (paySumGen>0 && gamesGen>0)
			 	outFile0 << (float)paySumGen/((float)gamesGen*8.0);
			outFile0 << ",";

			if (paidSumGen>0 && gamesGen>0)
				outFile0 << (float)paidSumGen/((float)gamesGen*2.0);
			outFile0 << ",";

			switch (runType)
			{
				case 0:
				{
					popStatsSummary(gen, &allAgents, deadGen, bornGen, choiceSize, g, outFile0);
					break;
				}
				case 1:
				{
					statsSummary(gen, &allAgents, deadGen, bornGen, choiceSize, g, outFile0, outFile1, outFile2, arcFile, socNet);
					break;
				}
			}
		//}
		} // Close critical pragma output_data

		//Cleans out know in each agent by removing those agents that have died.
		for (int i=0; i<(int)allAgents.size(); i++)
			(allAgents).at(i)->cleanKnow(&deadIDs, agentCount);

		//Wipe deadIDs to minimize size
		deadIDs.clear();

		//Breaks the population if it explodes or gets pointlessly small
		if (numGens-gen<=1  || (int)allAgents.size()>maxAgents || (int)allAgents.size()<=1 || (int)allAgents.size()==0)
		{
			/* no social networking is needed for the thesis so this is turned off
			if (socNet==true)
				for (int i=0; i<(int)allAgents.size(); i++)
					vertTimes.insert(make_pair((allAgents.at(i))->agentID(),make_pair((allAgents.at(i))->agentBirthday(),(allAgents.at(i))->agentBirthday() + (allAgents.at(i))->agentAge())));
			*/
			break;
		}

		//for (int i=0; i<(int)vertTimes.size(); i++)
		//pajFile << i+1 << " \"" << i+1 << "\" [" << vertTimes.at(i+1).first << "-" << vertTimes.at(i+1).second << "]\r" << endl;

	}  //Closes the loop that cycles through each GENERATION

	/* no social networking is needed for the thesis so this is turned off
	if (socNet==true)
	{
		//Create social network file
		pajFile << "*Vertices " << (int)vertTimes.size() << "\r" << endl;
		for (int i=1; i<(int)vertTimes.size()+1; i++)
			pajFile << i << " \"    ID" << i << "\" [" << vertTimes.at(i).first << "-" << vertTimes.at(i).second << "]\r" << endl;
	}
	*/

	//destroys vector and frees memory
	for (int i=0; i<(int)allAgents.size();i++)
		delete allAgents.at(i);
	allAgents.clear();

	/* no social networking is needed for the thesis so this is turned off
	//clear vertTimes for next population
	vertTimes.clear();
	*/

} //Closes for ne<numEnvs

	gsl_rng_free (r); //frees memory associated with random number generation

} //Closes for np<numPops

	trialLineNum++;  //update trial line number for next loop iteration

} //Closes WHILE loop that reads IN.DAT

	cout << "done" << endl;

	return 0;

} //Closes main


void prepOutfiles (ostream &outFile0, ostream &outFile1, ostream &outFile2, const int choiceSize, const int g, const int runType)
{
	//set precision and insert headers into summary stats collector
	outFile0 << setprecision( 5 ) << "\"TrialLine#\",\"Population#\",\"Environment#\",\"Game Type\",\"Round-Ante\",\"Spawn Cost\",\"Random Death\",\"Spawn Disc?\",\"Play Disc?\",\"Randomize Like?\",\"Randomize AfterPlay?\",";
	outFile0 << "\"Initial-#\",\"Total Generations\",\"Rounds/Gen\",\"Generation\",\"Avg. Payoff Off.\",\"Avg. Payoff Rec.\",\"#Agents-Dead\",";
	outFile0 << "\"#Agents-Born\",\"Births:Deaths\",\"Avg. Age\",\"Max Age\",\"#Max Age\",\"Avg. Bits\",\"#Agents-End\",";

	switch (runType)
	{
		case 0:
			break;
		case 1:
		{
			outFile0 << "\"Avg.# Known\", \"Avg. #Ns\",\"Avg. #Es\",\"Avg. #Fs\",\"Avg. #~Ns\",\"Avg. #~Es\",\"Avg. #~Fs\",";

			for (int i=0;i<3;i++)
			for (int j=0;j<3;j++)
			for (int k=0;k<3;k++)
			for (int l=0;l<3;l++)
			{
				switch (i)
				{
					case 0:
						outFile0 << "\"N\"" << j << k << l << ",";
						break;
					case 1:
						outFile0 << "\"E\"" << j << k << l << ",";
						break;
					case 2:
						outFile0 << "\"F\"" << j << k << l << ",";
						break;
				}
			}

			outFile0 << "\"SpEE\",\"SpEN\",\"SpEF\",\"SpNE\",\"SpNN\",\"SpNF\",\"SpFE\",\"SpFN\",\"SpFF\",";
			outFile0 << "\"PpEE\",\"PpEN\",\"PpEF\",\"PpNE\",\"PpNN\",\"PpNF\",\"PpFE\",\"PpFN\",\"PpFF\",";

			switch (g)
			{
				case 0:
				{
				for (int i=0; i<10; i++)
					outFile0 << "\"S\"" << i+1 << ",\"S\"" << i+1 << "%,";

				for (int i=0; i<10; i++)
					outFile0 << "\"NS\"" << i+1 << ",\"NS\"" << i+1 << "%,";
				}
				break;

				case 1:
				{
				for (int i=0;i<3;i++)
				for (int j=0;j<3;j++)
				for (int k=0;k<4;k++)
					outFile0<< "\"PD" << PDbitBase(k)+j*4096+i*12288 << "\",";
				}
				break;

				case 2:
				{
				for (int i=0;i<3;i++)
				for (int j=0;j<3;j++)
				for (int k=0;k<4;k++)
					outFile0<< "\"SH" << SHbitBase(k)+j*4096+i*12288 << "\",";
				}
				break;

				case 3:
				{
				for (int i=0;i<3;i++)
				for (int j=0;j<3;j++)
				for (int k=0;k<4;k++)
					outFile0<< "\"CK" << CKbitBase(k)+j*4096+i*12288 << "\",";
				}
				break;

				case 4:
				{
				for (int i=0;i<3;i++)
				for (int j=0;j<3;j++)
				for (int k=0;k<4;k++)
					outFile0<< "\"BS" << BSbitBase(k)+j*4096+i*12288 << "\",";
				}
				break;
		}

		//set precision and setup headers for the files that collect complete population information
		outFile1 << setprecision( 5 );
		outFile2 << setprecision( 5 );

		for (int i=0; i<choiceSize; i++)
			if (i <= 3280)
				outFile1 << "X" << pos2bit(i) << ",";
			else
				outFile2 << "X" << pos2bit(i) << ",";

		outFile1 << endl;
		outFile2 << endl;
		}
	} //closes switch(runType)
	outFile0 << endl;
}

int bit2pos(const int bit)
{
	int q1=0, q2=0, q3=0, q4=0, q5=0, q6=0;

	q1=bit/4;
	q2=bit/16;
	q3=bit/64;
	q4=bit/256;
	q5=bit/1024;
	q6=bit/4096;
	return bit-q1-q2*3-q3*9-q4*27-q5*81-q6*243;
}


int pos2bit(const int position)
{
	int q1=0, q2=0, q3=0, q4=0, q5=0, q6=0;

	q1=position/3;
	q2=position/9;
	q3=position/27;
	q4=position/81;
	q5=position/243;
	q6=position/729;
	return position+q1+q2*4+q3*16+q4*64+q5*256+q6*1024;
}


int PDbitBase(const int bitBase)
{
	switch(bitBase)
	{
		case 0:
			return 1386;
			break;
		case 1:
			return 1690;
			break;
		case 2:
			return 2405;
			break;
		case 3:
			return 2709;
			break;
	}
	return 666; //if returned, indicates error in switch
}

int SHbitBase(const int bitBase)
{
	switch(bitBase)
	{
		case 0:
			return 1170; //Mutual Defection, both hunt rabbits H, H
			break;
		case 1:
			return 1365; //Important Bit: Everyone does worse, both hunt stag  S, S
			break;
		case 2:
			return 2154; //agent unilaterally defects R, S
			break;
		case 3:
			return 2729; //agent unilaterally defected AGAINST  S, R
			break;
	}
	return 666; //if returned, indicates error in switch
}

int CKbitBase(const int bitBase)
{
	switch(bitBase)
	{
		case 0:
			return 1366; // agent wins at chicken St, Sw
			break;
		case 1:
			return 1429; // agent loses at chicken Sw, St
			break;
		case 2:
			return 2409; // both chicken out Sw, Sw
			break;
		case 3:
			return 2730; // CRASH St, St
			break;
	}
	return 666; //if returned, indicates error in switch
}

int BSbitBase(const int bitBase)
{
	switch(bitBase)
	{
		case 0:
			return 1366; // agent favourite W, W
			break;
		case 1:
			return 1429; // partner favourite O, O
			break;
		case 2:
			return 2665;  // Each at favourite alone W, O
			break;
		case 3:
			return 2730; // Ships in the night, each at non-favourite alone O, W
			break;
	}
	return 666; //if returned, indicates error in switch
}

void cull(vector<agent*> *allAgentsPtr, int &deadGenRef, const int anteRef, const float rndDeathRateRef, vector<int>* deadIDsPtr, map<unsigned int, pair<unsigned short, unsigned short> > *vertMapPtr, const bool socNet, gsl_rng * r)
{
	/* This function takes two inputs, a minimum threshold of resources which
	 * each agent must have acquired in order to survive to the next generation
	 * and a random distribution of death over the population as a whole, and
	 * based on these inputs it removes agents from future generations.
	*/

	//cout << "Beginning cull. ";

	deadGenRef=0;

	//create pair used to simplify the updating social map process
	pair<unsigned short, unsigned short> tempPair;

	//determine who lives and who dies
	for (int i=0; i<(int)allAgentsPtr->size(); i++)
	{
		//cout << "Testing agent: " << (allAgentsPtr->at(i))->agentID() << " at position: " << i << endl ;
		if ((allAgentsPtr->at(i))->agentRes() < anteRef || gsl_rng_uniform (r) <= rndDeathRateRef)
		{
			/*
			if ((allAgentsPtr->at(i))->agentRes() < anteRef)
				cout << "Agent " << (allAgentsPtr->at(i))->agentID() << " starves with resources: " << (allAgentsPtr->at(i))->agentRes() << endl;
			else
				cout << "Agent " << (allAgentsPtr->at(i))->agentID() << " dies randomly." << endl;
			*/

			deadIDsPtr->push_back((allAgentsPtr->at(i))->agentID());

			if (socNet==true)
			{
				tempPair.first=(allAgentsPtr->at(i))->agentBirthday();
				tempPair.second=(allAgentsPtr->at(i))->agentBirthday()+(allAgentsPtr->at(i))->agentAge();
				vertMapPtr->insert(make_pair((allAgentsPtr->at(i))->agentID(), make_pair(tempPair.first, tempPair.second)));
			}

			//cout << "agent at " << i << " prior delete: " << (allAgentsPtr->at(i))->agentID() << endl;
			delete allAgentsPtr->at(i); //Deletes the agent reffered to in allAgents
			//cout << "agent at " << i << " post delete: " << (allAgentsPtr->at(i))->agentID() << endl;
			allAgentsPtr->erase((allAgentsPtr->begin())+i); //Deletes the reference to the deleted agent in allAgents
			//cout << "agent at " << i << " post erase: " << (allAgentsPtr->at(i))->agentID() << endl;

			//deleting agents and references shifts agent positions in vector and so compensation is needed
			i = i - 1;

			deadGenRef++;
		}
		else
			(allAgentsPtr->at(i))->setRes((allAgentsPtr->at(i))->agentRes() - anteRef);
	}
}



void breed(vector<agent*> *allAgentsPtr, int &bornGenRef, const int spawnCostRef, const int spawnDisc, const int playDisc, const bool rndLike, const bool rndAft, const int birthPop, const int maxAbsent, gsl_rng * r, int &agentCountRef )
{
	//Spawn new agents based on remaining resources
	/* Reuse agentIndex vector.  Into this vector load the positions
	 * of each agent in allAgents with enough resources to spawn
	 * at least once.  If there are at least two agents in the vector
	 * then continue, otherwise start a new round or generation.
	 * Randomly match spawning agents, reduce their resources by the
	 * spawning cost, and create a new single agent that has each bit
	 * in its bitsets with 50% likelihood from either parent.  Continue
	 * until there are less than two agents remaining to be matched
	 * then start again.
	*/

	//cout << "Beginning spawn. ";

	vector<pair<unsigned long, unsigned short> > agentIndex;

	pair<unsigned long, unsigned short> tempPair;

	//cout << "allAgentsPtr->size() " << 	(int)allAgentsPtr->size() << endl;
	//cout << "agentIndex.size() prior to fill " << agentIndex.size() << endl;

	bornGenRef = 0;

	for (int i=0; i<(int)allAgentsPtr->size(); i++)
	{
		//cout << "Agent ID " << (allAgentsPtr->at(i))->agentID() << " Index no. " << i << " has resources " << (allAgentsPtr->at(i))->agentRes() << " and is ";
		if ((allAgentsPtr->at(i))->agentRes() >= spawnCostRef)
		{
			tempPair.first=i;
			tempPair.second=0;
			agentIndex.push_back(tempPair);
			//cout << "in" << endl;
		}
		//else
			//cout << "out" << endl;
	}

	//cout << "agentIndex.size() post fill " << agentIndex.size() << endl;

	unsigned int index1=0;
	unsigned int index2=0;

	while (agentIndex.size()>=2)
	{
		index1 = gsl_rng_uniform_int(r,agentIndex.size());
		index2 = gsl_rng_uniform_int(r,agentIndex.size());

		if (index1!=index2) //Prevents incrementing spawnAttempts if agent was set to spawn with themselves
		{
			agentIndex.at(index1).second++;
			agentIndex.at(index2).second++;

			if (spawnDisc<=0
				|| (spawnDisc>0
				&& ((allAgentsPtr->at(agentIndex.at(index1).first))->spawnAgree(allAgentsPtr->at(agentIndex.at(index2).first)->agentID())==true
				&& (allAgentsPtr->at(agentIndex.at(index2).first))->spawnAgree(allAgentsPtr->at(agentIndex.at(index1).first)->agentID())==true)))
			{
				//Withdraw spawning cost from selected agents
				//cout << (allAgentsPtr->at(agentIndex.at(index1)))->agentID() << " Index no. " << index1 << " has resources " << (allAgentsPtr->at(agentIndex.at(index1)))->agentRes() << endl;
				(allAgentsPtr->at(agentIndex.at(index1).first))->setRes((allAgentsPtr->at(agentIndex.at(index1).first))->agentRes()-spawnCostRef);
				//cout << "Agent resources post withdrawal " << (allAgentsPtr->at(agentIndex.at(index1)))->agentRes() << endl;
				(allAgentsPtr->at(agentIndex.at(index2).first))->setRes((allAgentsPtr->at(agentIndex.at(index2).first))->agentRes()-spawnCostRef);

				//Spawn a new agent based on Parents
				//Note: birthPop=gen+1
				allAgentsPtr->push_back(new agent(spawnDisc, playDisc, rndLike, rndAft, birthPop, maxAbsent, r, agentCountRef));
				(allAgentsPtr->back())->agentSpawn((allAgentsPtr->at(agentIndex.at(index1).first)),(allAgentsPtr->at(agentIndex.at(index2).first)),r);
				agentCountRef++;
				//cout << "A new agent is born!  ID# " << (allAgentsPtr->back())->agentID() << endl;

				bornGenRef++;
			}

			//Test just indexed agents to see if they qualify for further
			//spawning at this time.  If not, remove them from the index.
			if (index1 > index2)
			{
				if ((allAgentsPtr->at(agentIndex.at(index1).first))->agentRes() < spawnCostRef || agentIndex.at(index1).second >= spawnDisc)
				{
					//cout << "agent ID" << (allAgentsPtr->at(agentIndex.at(index1)))->agentID() << " removed" << endl;
					agentIndex.erase( agentIndex.begin() + index1);
				}
				if ((allAgentsPtr->at(agentIndex.at(index2).first))->agentRes() < spawnCostRef || agentIndex.at(index2).second >= spawnDisc)
				{
					//cout << "agent ID" << (allAgentsPtr->at(agentIndex.at(index2)))->agentID() << " removed" << endl;
					agentIndex.erase( agentIndex.begin() + index2);
				}
			}
			else
			{
				if ((allAgentsPtr->at(agentIndex.at(index2).first))->agentRes() < spawnCostRef || agentIndex.at(index2).second >= spawnDisc)
				{
					//cout << "agent ID" << (allAgentsPtr->at(agentIndex.at(index2)))->agentID() << " removed" << endl;
					agentIndex.erase( agentIndex.begin() + index2);
				}
				if ((allAgentsPtr->at(agentIndex.at(index1).first))->agentRes() < spawnCostRef || agentIndex.at(index1).second >= spawnDisc)
				{
					//cout << "agent ID" << (allAgentsPtr->at(agentIndex.at(index1)))->agentID() << " removed" << endl;
					agentIndex.erase( agentIndex.begin() + index1);
				}
			}
		}	//Closes if (index1!=index2)
	}	//Closes: while (agentIndex.size()>=2)
	agentIndex.clear();
}



void statsSummary(const int gen, vector<agent*> *allAgentsPtr, const int deadGen, const int bornGen, const int choiceSize, const int g, ostream &outFile0, ostream &outFile1, ostream &outFile2, ostream &arcFile, const bool socNet)
{
	//cout << "Running Stats Summary"<<endl;

	//Collect information across all agents
	/* Imporant information to collect across agents includes:
	 * 1. Total number of agents
	 * 2. Number of dead
	 * 3. Number born
	 * 4. Average payoff available across generation
	 * 5. Average payoff achieved across generation
	 * 6. Switch-Stay (1-0) Ratio across the population at end of generation
	 * 7. Total number of switches(stays) for each output across generation
	 * 8. Average agent age
	 * 9. afterPlay Averages across population
	 * 10. Oldest Agent & ID
	*/

	int maxAge=0;  						//age of oldest still living agent in generations survived
	int maxAgeCount=0; 					//Count of oldest living agents
	int bitsSetGen=0; 					//Sum of all set choice bits across a generation
	float avBitsSetGen=0;				//Average Number of bits set in the decision matrix across population
	int choiceTotals[choiceSize]; 		//Tracks total number of switches for each bit in choice across a generation. Initializes all elements to 0.
	float avAgeGen; 					//Average age of population: ageGen/allAgents.size()
	float afterPlayStats[3][3][3][3] = {{{{0}}}};	//Tracks afterPlay values across a population. Initializes to 0.
	float spawnPrefStats[9] = {0};		//Tracks spawnPref averages across population
	float playPrefStats[9] = {0};
	int likingCounts[6] = {0};
	int ageGen=0; 						//Sum of all living agent ages
	int agentsKnown=0;					//Track total number of agents known across population

	for (int i=0; i<choiceSize; i++)
		choiceTotals[i]=0;

	//Collect data for StatsSummary
	for (int i=0; i<(int)allAgentsPtr->size(); i++)
	{
		switch (g)
		{
			case 0:
				{
					for (int j=0; j<choiceSize; j++)
						if ((*allAgentsPtr).at(i)->choiceBitTest(j)==true)
							choiceTotals[j]++;
					break;
				}
			case 1:
				{
					for (int j=0;j<3;j++)
					for (int k=0;k<3;k++)
					for (int l=0;l<4;l++)
						if ((*allAgentsPtr).at(i)->choiceBitTest(bit2pos(PDbitBase(l)+k*4096+j*12288))==true)
							choiceTotals[bit2pos(PDbitBase(l)+k*4096+j*12288)]++;
				break;
				}
			case 2:
				{
					for (int j=0;j<3;j++)
					for (int k=0;k<3;k++)
					for (int l=0;l<4;l++)
						if ((*allAgentsPtr).at(i)->choiceBitTest(bit2pos(SHbitBase(l)+k*4096+j*12288))==true)
							choiceTotals[bit2pos(SHbitBase(l)+k*4096+j*12288)]++;
				break;
				}
			case 3:
				{
					for (int j=0;j<3;j++)
					for (int k=0;k<3;k++)
					for (int l=0;l<4;l++)
						if ((*allAgentsPtr).at(i)->choiceBitTest(bit2pos(CKbitBase(l)+k*4096+j*12288)==true))
							choiceTotals[bit2pos(CKbitBase(l)+k*4096+j*12288)]++;
				break;
				}
			case 4:
				{
					for (int j=0;j<3;j++)
					for (int k=0;k<3;k++)
					for (int l=0;l<4;l++)
						if ((*allAgentsPtr).at(i)->choiceBitTest(bit2pos(BSbitBase(l)+k*4096+j*12288)==true))
							choiceTotals[bit2pos(BSbitBase(l)+k*4096+j*12288)]++;
				break;
				}
		}

		bitsSetGen = bitsSetGen + (*allAgentsPtr).at(i)->agentBitsSet();
		ageGen = ageGen + (*allAgentsPtr).at(i)->agentAge();

		if ((	(*allAgentsPtr).at(i)->agentAge()) >= maxAge)
		{
			if ((	(*allAgentsPtr).at(i)->agentAge()) > maxAge)
			{
				maxAge = (*allAgentsPtr).at(i)->agentAge();
				maxAgeCount = 1;
			}
			else
				maxAgeCount++;
		}

		//Collect AfterPlay distribution statistics

		for (int j=0; j<3; j++)
		for (int k=0; k<3; k++)
		for (int l=0; l<3; l++)
		switch ((*allAgentsPtr).at(i)->afterPlayChar(j,k,l))
		{
			case 'N':
				afterPlayStats[0][j][k][l]++;
				break;
			case 'E':
				afterPlayStats[1][j][k][l]++;
				break;
			case 'F':
				afterPlayStats[2][j][k][l]++;
				break;
			default:
				break;
		}

		for (int j=0; j<9; j++)
			if ((*allAgentsPtr).at(i)->spawnPrefSet(j)==true)
				spawnPrefStats[j]++;

		for (int j=0; j<9; j++)
			if ((*allAgentsPtr).at(i)->playPrefSet(j)==true)
				playPrefStats[j]++;

		agentsKnown = agentsKnown+(*allAgentsPtr).at(i)->agentsKnown();

		for (int j=0; j<6; j++)
			likingCounts[j] += (*allAgentsPtr).at(i)->likingStatsCount(j);

		if (socNet==true)
		{
			for (int j=0; j<(int)allAgentsPtr->size(); j++)
			{
			if ((*allAgentsPtr).at(i)->agentKnow((*allAgentsPtr).at(j)->agentID())==true)
				{
					arcFile << (*allAgentsPtr).at(i)->agentID() << " ";
					arcFile << (*allAgentsPtr).at(j)->agentID() << " ";
					arcFile << (*allAgentsPtr).at(i)->agentLike((*allAgentsPtr).at(j)->agentID(), true);
					arcFile << " [" << gen+1 << "]\r" << endl;

					arcFile << (*allAgentsPtr).at(i)->agentID() << " ";
					arcFile << (*allAgentsPtr).at(j)->agentID() << " ";
					arcFile << (*allAgentsPtr).at(i)->agentLike((*allAgentsPtr).at(j)->agentID(), false)+1;
					arcFile << " [" << gen+1 << "]\r" << endl;
				}
			}
		}

	}  //Closes for i<(int)allAgentsPtr->size()

	avAgeGen = (float)ageGen/(float)allAgentsPtr->size();
	//avBitsSetGen = allAgentsPtr->size();
	avBitsSetGen = (float)bitsSetGen/((float)allAgentsPtr->size()*choiceSize);

	outFile0 << deadGen << "," << bornGen << ",";
	if (deadGen>0 && bornGen>0)
		outFile0 << (float)bornGen/(float)deadGen;

	outFile0 << ",";

	if ((float)allAgentsPtr->size() > 0)
		outFile0 << avAgeGen;

	outFile0 << "," << maxAge << ",";
	outFile0 << maxAgeCount << ",";

	if ((float)allAgentsPtr->size() > 0)
		outFile0 << avBitsSetGen;

	outFile0 << "," << allAgentsPtr->size() << ",";

	if ((float)allAgentsPtr->size() > 0)
		outFile0 << (float)agentsKnown/(float)allAgentsPtr->size();

	outFile0 << ",";

	for (int j=0; j<6; j++)
	{
		if ((float)allAgentsPtr->size() > 0)
			outFile0 << (float)likingCounts[j]/(float)allAgentsPtr->size();

		outFile0 << ",";
	}


	//Output AfterPlay Statistics after first averaging them

	for (int h=0; h<3; h++)
	{
		for (int j=0; j<3; j++)
		for (int k=0; k<3; k++)
		for (int l=0; l<3; l++)
		{
			if ((float)allAgentsPtr->size() > 0)
				outFile0 << afterPlayStats[h][j][k][l]/(float)allAgentsPtr->size();

			outFile0 << ",";
		}
	}

	//Output spawnPref statistics after averaging
	for (int j=0; j<9; j++)
	{
		if ((float)allAgentsPtr->size() > 0)
			outFile0 << spawnPrefStats[j]/(float)allAgentsPtr->size();

		outFile0 << ",";
	}

	//Output playPref averages
	for (int j=0; j<9; j++)
	{
		if ((float)allAgentsPtr->size() > 0)
			outFile0 << playPrefStats[j]/(float)allAgentsPtr->size();

		outFile0 << ",";
	}

	//DUMP ALL SENSIBLE BIT STATS TO OUTFILE1 and OUTFILE2
	int junkCounter=0; 					//BRIEFLY holds a value to check a counter against
	float junkValue=0.0; 				//BRIEFLY holds a value for copying/shifting/etc.
	vector<float> topBitProportions(10, 0.0);	//Stores the population averages for the highest/lowest set bits
	vector<int> topBitReferences(10, 0);		//Stores the bitset loactions of most frequently (un)set bits

	switch (g)
	{
		case 0:
			{
				//Output top ten bits with the highest percentage set

				for (int j=0; j<choiceSize; j++)
				{
					junkValue=(float)choiceTotals[j]/(float)allAgentsPtr->size();

					junkCounter=0;
					while (junkCounter<10 && junkValue < topBitProportions.at(junkCounter))
						junkCounter++;

					if (junkCounter<10)
					{
						topBitProportions.insert(find(topBitProportions.begin(),topBitProportions.end(),topBitProportions.at(junkCounter)),junkValue);
						topBitProportions.pop_back();

						topBitReferences.insert(find(topBitReferences.begin(),topBitReferences.end(),topBitReferences.at(junkCounter)),j);
						topBitReferences.pop_back();
					}

				}


				for (int j=0; j<(int)topBitProportions.size(); j++)
				{
					outFile0 << pos2bit(topBitReferences.at(j))+1 << "," << topBitProportions.at(j) << ",";
				}

				//Output top ten bits with the lowest percentage set

				for (int j=0; j<10; j++)
				{
					topBitProportions.at(j)=0.0;
					topBitReferences.at(j)=0;
				}

				for (int j=0; j<choiceSize; j++)
				{

					junkValue=1.0-((float)choiceTotals[j]/(float)allAgentsPtr->size());

					junkCounter=0;
					while (junkCounter<10 && junkValue < topBitProportions.at(junkCounter))
						junkCounter++;

					if (junkCounter<10)
					{
						topBitProportions.insert(find(topBitProportions.begin(),topBitProportions.end(),topBitProportions.at(junkCounter)),junkValue);
						topBitProportions.pop_back();

						topBitReferences.insert(find(topBitReferences.begin(),topBitReferences.end(),topBitReferences.at(junkCounter)),j);
						topBitReferences.pop_back();
					}

				}

				for (int j=0; j<(int)topBitProportions.size(); j++)
					outFile0 << pos2bit(topBitReferences.at(j))+1 << "," << 1.0-topBitProportions.at(j) << ",";

			//Dump ALL bits to X-Files
			for (int j=0; j<choiceSize; j++)
			{
				if (j <= 3280)
				{
					if ((float)allAgentsPtr->size() > 0)
						outFile1 << (float)choiceTotals[j]/(float)allAgentsPtr->size();

					outFile1 << ",";
				}
				else
				{
					if ((float)allAgentsPtr->size() > 0)
						outFile2 << (float)choiceTotals[j]/(float)allAgentsPtr->size();

					outFile2 << ",";
				}
			}

			outFile1 << endl;
			outFile2 << endl;

			topBitReferences.clear();
			topBitProportions.clear();
			}
			break;
		case 1:
			{
				for (int i=0;i<3;i++)
				for (int j=0;j<3;j++)
				for (int k=0;k<4;k++)
				{
					if ((float)allAgentsPtr->size() > 0)
						outFile0<< (float)choiceTotals[bit2pos(PDbitBase(k)+j*4096+i*12288)]/(float)allAgentsPtr->size();
					outFile0 << ",";
				}
			}
			break;
		case 2:
			{
				for (int i=0;i<3;i++)
				for (int j=0;j<3;j++)
				for (int k=0;k<4;k++)
				{
					if ((float)allAgentsPtr->size() > 0)
						outFile0<< (float)choiceTotals[bit2pos(SHbitBase(k)+j*4096+i*12288)]/(float)allAgentsPtr->size();
					outFile0 << ",";
				}
			}
			break;
		case 3:
			{
				for (int i=0;i<3;i++)
				for (int j=0;j<3;j++)
				for (int k=0;k<4;k++)
				{
					if ((float)allAgentsPtr->size() > 0)
						outFile0<< (float)choiceTotals[bit2pos(CKbitBase(k)+j*4096+i*12288)]/(float)allAgentsPtr->size();
					outFile0 << ",";
				}
			}
			break;
		case 4:
			{
				for (int i=0;i<3;i++)
				for (int j=0;j<3;j++)
				for (int k=0;k<4;k++)
				{
					if ((float)allAgentsPtr->size() > 0)
						outFile0<< (float)choiceTotals[bit2pos(BSbitBase(k)+j*4096+i*12288)]/(float)allAgentsPtr->size();
					outFile0 << ",";
				}
			}
			break;
	}
	outFile0 << endl; //Nothing else to put in outFile0 so end the line.

}

void popStatsSummary(const int gen, vector<agent*> *allAgentsPtr, const int deadGen, const int bornGen, const int choiceSize, const int g, ostream &outFile0)
{
	//cout << "Running Short Stats Summary"<<endl;


	int maxAge=0;  						//age of oldest still living agent in generations survived
	int maxAgeCount=0; 					//Count of oldest living agents
	int bitsSetGen=0; 					//Sum of all set choice bits across a generation
	float avBitsSetGen=0;				//Average Number of bits set in the decision matrix across population
	float avAgeGen; 					//Average age of population: ageGen/allAgents.size()
	int ageGen=0; 						//Sum of all living agent ages
	int agentsKnown=0;					//Track total number of agents known across population


	for (int i=0; i<(int)allAgentsPtr->size(); i++)
	{
		bitsSetGen = bitsSetGen + (*allAgentsPtr).at(i)->agentBitsSet();
		ageGen = ageGen + (*allAgentsPtr).at(i)->agentAge();

		if ((	(*allAgentsPtr).at(i)->agentAge()) >= maxAge)
		{
			if ((	(*allAgentsPtr).at(i)->agentAge()) > maxAge)
			{
				maxAge = (*allAgentsPtr).at(i)->agentAge();
				maxAgeCount = 1;
			}
			else
				maxAgeCount++;
		}

		agentsKnown = agentsKnown+(*allAgentsPtr).at(i)->agentsKnown();

	}  //Closes for i<(int)allAgentsPtr->size()

	avAgeGen = (float)ageGen/(float)allAgentsPtr->size();
	//avBitsSetGen = allAgentsPtr->size();
	avBitsSetGen = (float)bitsSetGen/((float)allAgentsPtr->size()*choiceSize);

	outFile0 << deadGen << "," << bornGen << ",";
	if (deadGen>0 && bornGen>0)
		outFile0 << (float)bornGen/(float)deadGen;

	outFile0 << ",";

	if ((float)allAgentsPtr->size() > 0)
		outFile0 << avAgeGen;

	outFile0 << "," << maxAge << ",";
	outFile0 << maxAgeCount << ",";

	if ((float)allAgentsPtr->size() > 0)
		outFile0 << avBitsSetGen;

	outFile0 << "," << allAgentsPtr->size();

	outFile0 << endl;
}
